/*
  frequency scale
*/
#include <stdlib.h>
#include <math.h>
#include "sbr_def.h"
#include "sbr.h"

int getStartFreq(int fs, int start_freq);
int getStopFreq(int fs, int stop_freq);

static inline int numberOfBands(int b_p_o, int start, int stop, float warp_factor)
{
  int result=0;
  result = 2* (int) (b_p_o * logf((float) (stop)/start) / (2.0f*M_LN2*warp_factor) +0.5);
  return(result);
}

static float _expf(float x)
{       
        float P1 =  1.6666667163e-01; 
        float P2 = -2.7777778450e-03; 
        float c,t;
        t  = x*x;
        c  = x - t*(P1+t*P2);
        return 1.0f-((x*c)/(c-2.0f)-x);
}

static void CalcBands(int * diff, int start, int stop, int num_bands)
{
  int i;
  int previous;
  int current;
  previous=start;
  for(i=1; i<= num_bands; i++) {
      //current=(int)((start * powf((float)stop/start, (float)i/num_bands)) + 0.5f);
      current=(int)((start*_expf((float)i/num_bands*logf((float)stop/start))) + 0.5f);
      diff[i-1]=current-previous;
      previous=current;
  }
}

static inline void cumSum(int start_value, int* diff, int length,  unsigned char *start_adress)
{
  int i;
  start_adress[0]=start_value;
  for(i=1;i<=length;i++) {
    start_adress[i]=start_adress[i-1]+diff[i-1];
  }
}

static inline int modifyBands(int max_band_previous, int * diff, int length)
{
  int change=max_band_previous-diff[0];
  if (change > (diff[length-1] - diff[0]) / 2) {
    change = (diff[length-1] - diff[0]) / 2;
  }
  diff[0] += change;
  diff[length-1] -= change;
  Shellsort_int(diff, length);
  return(0);
}

int getStartFreq(int fs, int start_freq)
{
  int k0_min;

  switch(fs){
  case 16000: k0_min = 24;
    break;
  case 22050: k0_min = 17;
    break;
  case 24000: k0_min = 16;
    break;
  case 32000: k0_min = 16;
    break;
  case 44100: k0_min = 12;
    break;
  case 48000: k0_min = 11;
    break;
  default:
    k0_min=11; /* illegal fs */
  }

  switch (fs) {
  case 16000:
    {
      int v_offset[]= {-8, -7, -6, -5, -4, -3, -2, -1, 0, 1, 2, 3, 4, 5, 6, 7};
      return (k0_min + v_offset[start_freq]);
    }
    break;

  case 22050:
    {
      int v_offset[]= {-5, -4, -3, -2, -1, 0, 1, 2, 3, 4, 5, 6, 7, 9, 11, 13};
      return (k0_min + v_offset[start_freq]);
    }
    break;

  case 24000:
    {
      int v_offset[]= {-5, -3, -2, -1, 0, 1, 2, 3, 4, 5, 6, 7, 9, 11, 13, 16};
      return (k0_min + v_offset[start_freq]);
    }
    break;

  case 32000:
    {
      int v_offset[]= {-6, -4, -2, -1, 0, 1, 2, 3, 4, 5, 6, 7, 9, 11, 13, 16};
      return (k0_min + v_offset[start_freq]);
    }
    break;

  case 44100:
  case 48000:
    {
      int v_offset[]= {-4, -2, -1, 0, 1, 2, 3, 4, 5, 6, 7, 9, 11, 13, 16, 20};
      return (k0_min + v_offset[start_freq]);
    }
    break;

  default:
    {
      int v_offset[]= {0, 1, 2, 3, 4, 5, 6, 7, 9, 11, 13, 16, 20, 24, 28, 33};
      return (k0_min + v_offset[start_freq]);
    }
  }
}

int getStopFreq(int fs, int stop_freq)
{
  int result,i;
  int *v_stop_freq = NULL;
  int k1_min;
  int v_dstop[13];

  int v_stop_freq_32[14] =  {32,34,36,38,40,42,44,46,49,52,55,58,61,64};
  int v_stop_freq_44[14] =  {23,25,27,29,32,34,37,40,43,47,50,55,59,64};
  int v_stop_freq_48[14] =  {21,23,25,27,30,32,35,38,42,45,49,54,59,64};

  switch(fs){
  case 22050:
  case 32000:
    k1_min = 32;
    v_stop_freq =v_stop_freq_32;
    break;

  case 44100:
    k1_min = 23;
    v_stop_freq =v_stop_freq_44;
    break;

  case 48000:
    k1_min = 21;
    v_stop_freq =v_stop_freq_48;
    break;

  default:
    k1_min = 21; /* illegal fs  */
  }

  for(i = 0; i <= 12; i++) {
    v_dstop[i] = v_stop_freq[i+1] - v_stop_freq[i];
  }

  Shellsort_int(v_dstop, 13);

  result = k1_min;

  for(i = 0; i < stop_freq; i++) {
    result = result + v_dstop[i];
  }

  return result;
}

int FindStartAndStopBand(const int samplingFreq, const int startFreq, const int stopFreq,
			 const SR_MODE sampleRateMode, int *k0, int *k2)
{
  *k0 = getStartFreq(samplingFreq, startFreq);
  if((sampleRateMode == 1) &&
     (samplingFreq*QMF_CHANNELS < 2**k0 * samplingFreq)) {
    return (1);
  }
  if (stopFreq < 14) {
    *k2 = getStopFreq(samplingFreq, stopFreq);
  } else {
      if(stopFreq == 14) {
          *k2 = 2 * *k0;
      } else {
          *k2 = 3 * *k0;
      }
  }
  if (*k2 > QMF_CHANNELS) {
    *k2 = QMF_CHANNELS;
  }
  //fprintf(stderr, "k0 = %i, k2 = %i, noChannels = %i\n", *k0, *k2, QMF_CHANNELS);
  if (*k2 - *k0 > QMF_CHANNELS / 2 - 4) {
    return (1);
  }
  if (*k2 > QMF_CHANNELS - 2) {
    return (1);
  }
  if ((*k2 - *k0) > MAX_FREQ_COEFFS) {
    return (1);
  }
  if ((*k2 - *k0) < 0) {
    return (1);
  }
  return(0);
}

int UpdateFreqScale(unsigned char  *v_k_master, int *h_num_bands, const int k0, const int k2,
		    const int freqScale, const int alterScale)
{

  int     b_p_o = 0;
  float   warp = 0;
  int     dk = 0;

  /* Internal variables */
  int     k1 = 0, i;
  int     num_bands0;
  int     num_bands1;
  int     diff_tot[MAX_OCTAVE + MAX_SECOND_REGION];
  int     *diff0 = diff_tot;
  int     *diff1 = diff_tot+MAX_OCTAVE;
  int     k2_achived;
  int     k2_diff;
  int     incr = 0;

  if(freqScale==1) {
    b_p_o=12;
  }

  if(freqScale==2) {
    b_p_o=10;
  }

  if(freqScale==3) {
    b_p_o=8;
  }

  if(freqScale > 0) {
      if(alterScale==0) {
        warp=1.0;
      } else {
        warp=1.3f;
      }

      if(4*k2 >= 9*k0) {
          k1=2*k0;
          num_bands0=numberOfBands(b_p_o, k0, k1, 1.0);
          num_bands1=numberOfBands(b_p_o, k1, k2, warp);
          CalcBands(diff0, k0, k1, num_bands0);
          Shellsort_int( diff0, num_bands0);

          if (diff0[0] == 0) {
            return (1);
          }

          cumSum(k0, diff0, num_bands0, v_k_master);
          CalcBands(diff1, k1, k2, num_bands1);
          Shellsort_int( diff1, num_bands1);

          if(diff0[num_bands0-1] > diff1[0]) {
              if(modifyBands(diff0[num_bands0-1],diff1, num_bands1)) {
                return(1);
              }
          }

          cumSum(k1, diff1, num_bands1, &v_k_master[num_bands0]);
          *h_num_bands=num_bands0+num_bands1;
      } else {
          k1=k2;
          num_bands0=numberOfBands(b_p_o, k0, k1, 1.0);
          CalcBands(diff0, k0, k1, num_bands0);
          Shellsort_int( diff0, num_bands0);
          if (diff0[0] == 0) {
            return (1);
          }
          cumSum(k0, diff0, num_bands0, v_k_master);
          *h_num_bands=num_bands0;
        }
    } else {
      if (alterScale==0) {
        dk = 1;
        num_bands0 = 2 * ((k2 - k0)/2);
      } else {
        dk = 2;
        num_bands0 = 2 * (((k2 - k0)/dk +1)/2);
      }
      k2_achived = k0 + num_bands0*dk;
      k2_diff = k2 - k2_achived;
      for(i=0;i<num_bands0;i++) {
        diff_tot[i] = dk;
      }
      if (k2_diff < 0) {
          incr = 1;
          i = 0;
      }
      if (k2_diff > 0) {
          incr = -1;
          i = num_bands0-1;
      }
      while (k2_diff != 0) {
        diff_tot[i] = diff_tot[i] - incr;
        i = i + incr;
        k2_diff = k2_diff + incr;
      }
      cumSum(k0, diff_tot, num_bands0, v_k_master);
      *h_num_bands=num_bands0;
    }
  if (*h_num_bands < 1) {
    return(1);
  }
  return (0);
}

