/*
  Sbr QMF inverse filtering detector
*/
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "ton_corr.h"
#define MAX_NUM_REGIONS 10

DETECTOR_PARAMETERS detectorParamsAAC = {
    {
      {INVF_MID_LEVEL,   INVF_LOW_LEVEL,  INVF_OFF,        INVF_OFF, INVF_OFF},
      {INVF_MID_LEVEL,   INVF_LOW_LEVEL,  INVF_OFF,        INVF_OFF, INVF_OFF},
      {INVF_HIGH_LEVEL,  INVF_MID_LEVEL,  INVF_LOW_LEVEL,  INVF_OFF, INVF_OFF},
      {INVF_HIGH_LEVEL,  INVF_HIGH_LEVEL, INVF_MID_LEVEL,  INVF_OFF, INVF_OFF},
      {INVF_HIGH_LEVEL,  INVF_HIGH_LEVEL, INVF_MID_LEVEL,  INVF_OFF, INVF_OFF}
    },
    {
      {INVF_LOW_LEVEL,   INVF_LOW_LEVEL,  INVF_LOW_LEVEL,  INVF_OFF, INVF_OFF},
      {INVF_LOW_LEVEL,   INVF_LOW_LEVEL,  INVF_LOW_LEVEL,  INVF_OFF, INVF_OFF},
      {INVF_HIGH_LEVEL,  INVF_MID_LEVEL,  INVF_MID_LEVEL,  INVF_OFF, INVF_OFF},
      {INVF_HIGH_LEVEL,  INVF_HIGH_LEVEL, INVF_MID_LEVEL,  INVF_OFF, INVF_OFF},
      {INVF_HIGH_LEVEL,  INVF_HIGH_LEVEL, INVF_MID_LEVEL,  INVF_OFF, INVF_OFF}
    },
};

DETECTOR_PARAMETERS detectorParamsAACSpeech = {
    {
      {INVF_MID_LEVEL,   INVF_MID_LEVEL,  INVF_LOW_LEVEL,  INVF_OFF, INVF_OFF},
      {INVF_MID_LEVEL,   INVF_MID_LEVEL,  INVF_LOW_LEVEL,  INVF_OFF, INVF_OFF},
      {INVF_HIGH_LEVEL,  INVF_MID_LEVEL,  INVF_MID_LEVEL,  INVF_OFF, INVF_OFF},
      {INVF_HIGH_LEVEL,  INVF_HIGH_LEVEL, INVF_MID_LEVEL,  INVF_OFF, INVF_OFF},
      {INVF_HIGH_LEVEL,  INVF_HIGH_LEVEL, INVF_MID_LEVEL,  INVF_OFF, INVF_OFF}
    },
    {
      {INVF_MID_LEVEL,   INVF_MID_LEVEL,  INVF_LOW_LEVEL,  INVF_OFF, INVF_OFF},
      {INVF_MID_LEVEL,   INVF_MID_LEVEL,  INVF_LOW_LEVEL,  INVF_OFF, INVF_OFF},
      {INVF_HIGH_LEVEL,  INVF_MID_LEVEL,  INVF_MID_LEVEL,  INVF_OFF, INVF_OFF},
      {INVF_HIGH_LEVEL,  INVF_HIGH_LEVEL, INVF_MID_LEVEL,  INVF_OFF, INVF_OFF},
      {INVF_HIGH_LEVEL,  INVF_HIGH_LEVEL, INVF_MID_LEVEL,  INVF_OFF, INVF_OFF}
    },
};

typedef const float FIR_FILTER[5];

FIR_FILTER fir_0 = { 1.0f };
FIR_FILTER fir_1 = { 0.3333333f, 0.6666666f };
FIR_FILTER fir_2 = { 0.125f, 0.375f, 0.5f };
FIR_FILTER fir_3 = { 0.0585786f, 0.2f, 0.3414214f, 0.4f };
FIR_FILTER fir_4 = { 0.0318305f, 0.1151638f, 0.2181695f, 0.3015028f, 0.3333333f };

FIR_FILTER *fir_table[5] = {
  &fir_0,
  &fir_1,
  &fir_2,
  &fir_3,
  &fir_4
};

static __attribute__((always_inline)) void calculateDetectorValues(float **quotaMatrixOrig, char * indexVector, float* nrgVector, DETECTOR_VALUES *detectorValues,
			     int startChannel, int stopChannel)
{
  int i,j;
  float origQuota, sbrQuota;
  const float* filter = *fir_table[INVF_SMOOTHING_LENGTH];
  float quotaVecOrig[64], quotaVecSbr[64];
  memset(quotaVecOrig,0,64*sizeof(float));
  memset(quotaVecSbr ,0,64*sizeof(float));
  detectorValues->avgNrg = 0;

  for(j = 0; j < 2; j++){
    for(i = startChannel; i < stopChannel; i++) {
      quotaVecOrig[i] += (quotaMatrixOrig[j][i]);
      if(indexVector[i] != -1) {
        quotaVecSbr[i] += (quotaMatrixOrig[j][indexVector[i]]);
      }
    }
    detectorValues->avgNrg += nrgVector[j];
  }

  detectorValues->avgNrg /= 2;

  for(i = startChannel; i < stopChannel; i++) {
    quotaVecOrig[i] /= 2;
    quotaVecSbr[i]  /= 2;
  }

  origQuota = 0.0f;
  sbrQuota  = 0.0f;

  for(i = startChannel; i < stopChannel; i++) {
    origQuota += quotaVecOrig[i];
    sbrQuota  += quotaVecSbr[i];
  }

  origQuota /= (stopChannel - startChannel);
  sbrQuota  /= (stopChannel - startChannel);

  memmove(detectorValues->origQuotaMean, detectorValues->origQuotaMean + 1, INVF_SMOOTHING_LENGTH*sizeof(float));
  memmove(detectorValues->sbrQuotaMean, detectorValues->sbrQuotaMean + 1, INVF_SMOOTHING_LENGTH*sizeof(float));

  detectorValues->origQuotaMean[INVF_SMOOTHING_LENGTH]          = origQuota;
  detectorValues->sbrQuotaMean[INVF_SMOOTHING_LENGTH]           = sbrQuota;
  detectorValues->origQuotaMeanFilt = 0;
  detectorValues->sbrQuotaMeanFilt = 0;

  for(i=0;i<INVF_SMOOTHING_LENGTH+1;i++) {
    detectorValues->origQuotaMeanFilt += detectorValues->origQuotaMean[i]*filter[i];
    detectorValues->sbrQuotaMeanFilt  += detectorValues->sbrQuotaMean[i]*filter[i];
  }
}

int findRegion(float currVal, const float* borders)
{
  int i;

  if(currVal < borders[0]) {
    return 0;
  }

  for(i = 1; i < 4; i++){
    if(currVal >= borders[i-1] && currVal < borders[i]) {
      return i;
    }
  }

  if(currVal > borders[3]) {
    return 4;
  }

  return 0;
}

/**************************************************************************
  \brief     Makes a decision on which invf mode to use,
	     based on the quota vector.
**************************************************************************/
static INVF_MODE decisionAlgorithm(const DETECTOR_PARAMETERS* detectorParams, DETECTOR_VALUES detectorValues,
			    int transientFlag, INVF_MODE prevInvfMode, int* prevRegionSbr, int* prevRegionOrig)
{
  int invFiltLevel, regionSbr, regionOrig, regionNrg;

  float quantStepsSbr[4]  = {1, 10, 14, 19};
  float quantStepsOrig[4] = {0,  3,  7, 10};
  float nrgBorders[4] =     {25.0f, 30.0f, 35.0f, 40.0f};

  float quantStepsSbrTmp[MAX_NUM_REGIONS];
  float quantStepsOrigTmp[MAX_NUM_REGIONS];

  float origQuotaMeanFilt;
  float sbrQuotaMeanFilt;
  float nrg;

  origQuotaMeanFilt = (float) (I_LN2*3.0*logf(detectorValues.origQuotaMeanFilt+EPS));
  sbrQuotaMeanFilt  = (float) (I_LN2*3.0*logf(detectorValues.sbrQuotaMeanFilt+EPS));
  nrg               = (float) (I_LN2*1.5*logf(detectorValues.avgNrg+EPS));

  memcpy(quantStepsSbrTmp,quantStepsSbr,4*sizeof(float));
  memcpy(quantStepsOrigTmp,quantStepsOrig,4*sizeof(float));

  if(*prevRegionSbr < 4) {
    quantStepsSbrTmp[*prevRegionSbr] = quantStepsSbr[*prevRegionSbr] + 1.0f;
  }

  if(*prevRegionSbr > 0) {
    quantStepsSbrTmp[*prevRegionSbr - 1] = quantStepsSbr[*prevRegionSbr - 1] - 1.0f;
  }

  if(*prevRegionOrig < 4) {
    quantStepsOrigTmp[*prevRegionOrig] = quantStepsOrig[*prevRegionOrig] + 1.0f;
  }

  if(*prevRegionOrig > 0) {
    quantStepsOrigTmp[*prevRegionOrig - 1] = quantStepsOrig[*prevRegionOrig - 1] - 1.0f;
  }

  regionSbr  = findRegion(sbrQuotaMeanFilt, quantStepsSbrTmp);
  regionOrig = findRegion(origQuotaMeanFilt, quantStepsOrigTmp);
  regionNrg  = findRegion(nrg,nrgBorders);

  *prevRegionSbr = regionSbr;
  *prevRegionOrig = regionOrig;

  if(transientFlag == 1){
    invFiltLevel = detectorParams->regionSpaceTransient[regionSbr][regionOrig];
  } else{
    invFiltLevel = detectorParams->regionSpace[regionSbr][regionOrig];
  }

  invFiltLevel = max(invFiltLevel + regionNrg - 4, 0);
  return (INVF_MODE) (invFiltLevel);
}

/**************************************************************************
  \brief     Estiamtion of the inverse filtering level required
             in the decoder.
**************************************************************************/
void InverseFilteringDetectorQmf (SBR_INV_FILT_EST *hInvFilt, float ** quotaMatrix, float *nrgVector, char* indexVector,
				  int transientFlag, INVF_MODE* infVec)
{
  int band;
  for(band = 0 ; band < hInvFilt->noDetectorBands; band++){
    int startChannel = hInvFilt->freqBandTableInvFilt[band];
    int stopChannel  = hInvFilt->freqBandTableInvFilt[band+1];
    calculateDetectorValues(quotaMatrix, indexVector, nrgVector, &hInvFilt->detectorValues[band], startChannel, stopChannel);
    infVec[band]= decisionAlgorithm(hInvFilt->detectorParams, hInvFilt->detectorValues[band], transientFlag, hInvFilt->prevInvfMode[band],
				   &hInvFilt->prevRegionSbr[band], &hInvFilt->prevRegionOrig[band]);
  }
}

int CreateInvFiltDetector (SBR_INV_FILT_EST *hInvFilt, int* freqBandTableDetector, int numDetectorBands, unsigned int useSpeechConfig)
{
  int i;
  memset( hInvFilt,0,sizeof(SBR_INV_FILT_EST));
  if (useSpeechConfig) {
    hInvFilt->detectorParams=&detectorParamsAACSpeech;
  } else {
    hInvFilt->detectorParams=&detectorParamsAAC;
  }
  hInvFilt->noDetectorBandsMax = numDetectorBands;
  for(i=0;i<hInvFilt->noDetectorBandsMax;i++){
    memset(&hInvFilt->detectorValues[i],0,sizeof(DETECTOR_VALUES));
    hInvFilt->prevInvfMode[i]   = INVF_OFF;
    hInvFilt->prevRegionOrig[i] = 0;
    hInvFilt->prevRegionSbr[i]  = 0;
  }
  memcpy(hInvFilt->freqBandTableInvFilt,freqBandTableDetector,(numDetectorBands+1)*sizeof(int));
  hInvFilt->noDetectorBands = numDetectorBands;
  return (0);
}

