/*
  Missing harmonics detection
*/
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "ton_corr.h" 
#define TIME_SLOTS 16

static unsigned char sbr_detectionVectors[MAX_CHANNELS*NO_OF_ESTIMATES*MAX_FREQ_COEFFS];
#define noEstPerFrame 2
#define totNoEst NO_OF_ESTIMATES

static void diff(float* pTonalityOrig, float* pDiffMapped2Scfb,
          const unsigned char* pFreqBandTable, int nScfb, char * indexVector)
{

  int i, ll, lu, k;
  float maxValOrig, maxValSbr;

  for(i=0; i < nScfb; i++){
    ll = pFreqBandTable[i];
    lu = pFreqBandTable[i+1];

    maxValOrig = 0;
    maxValSbr = 0;

    for(k=ll;k<lu;k++){
      if(pTonalityOrig[k] > maxValOrig) {
        maxValOrig = pTonalityOrig[k];
      }

      if(pTonalityOrig[indexVector[k]] > maxValSbr) {
        maxValSbr = pTonalityOrig[indexVector[k]];
      }
    }

    if(maxValSbr >= 1) {
      pDiffMapped2Scfb[i] = maxValOrig/maxValSbr;
    } else {
      pDiffMapped2Scfb[i] = maxValOrig;
    }
  }
}

/**************************************************************************/
/*!
  \brief     Calculates spectral flatness (ratio of geometric to arithmetic means),
			 a low spectral flatness corresponds to a highly tonal signal, whereas
			 a high spectral flatness indicates either silence or something noise-like
			 in character, in the sense that signals with approximately equivalent
			 energy levels in most/all spectral bands will sound similar to white noise,
			 which itsself has a perfectly flat power spectral density
			 (and thus a spectral flatness of \infty).	
*/
/**************************************************************************/
void calculateFlatnessMeasure(float* pQuotaBuffer,
                                     char* indexVector,
                                     float* pSfmOrigVec,
                                     float* pSfmSbrVec,
                                     const unsigned char* pFreqBandTable,
                                     int nSfb)
{
  int i,j;

  for(i=0;i<nSfb;i++) {
    int ll = pFreqBandTable[i];
    int lu = pFreqBandTable[i+1];
    int tmp = lu-ll;

    pSfmOrigVec[i] = 1;
    pSfmSbrVec[i]  = 1;

    if(lu -ll > 1){
      float amOrig,amTransp,gmOrig,gmTransp,sfmOrig,sfmTransp;

      amOrig = amTransp = 0;
      gmOrig = gmTransp = 1;

      for(j= ll;j<lu;j++){
        sfmOrig   = pQuotaBuffer[j];
        sfmTransp = pQuotaBuffer[indexVector[j]];
        amOrig   += sfmOrig;
        gmOrig   *= sfmOrig;
        amTransp += sfmTransp;
        gmTransp *= sfmTransp;
      }

      amOrig   /= lu-ll;
      amTransp /= lu-ll;

      //gmOrig   = (float) powf(gmOrig, 1.0f/(lu-ll));
      //gmTransp = (float) powf(gmTransp,1.0f/(lu-ll));
      switch(tmp) {
		case 1:
		    break;
		case 2:
			gmOrig = sqrtf(gmOrig);
			gmTransp = sqrtf(gmTransp);
			break;
		case 3:
			gmOrig = cbrtf(gmOrig);
			gmTransp = cbrtf(gmTransp);
		case 4:
		default:
			gmOrig = sqrtf(sqrtf(gmOrig));
			gmTransp = sqrtf(sqrtf(gmTransp));
			break;
      }

      if(amOrig) {
        pSfmOrigVec[i] = gmOrig/amOrig;
      }

      if(amTransp) {
        pSfmSbrVec[i] = gmTransp/amTransp;
      }
    }
  }
}

static void calculateDetectorInput(float** pQuotaBuffer,
                                   char* indexVector,
                                   float tonalityDiff[4][27],
                                   float pSfmOrig[4][27],
                                   float pSfmSbr[4][27],
                                   const unsigned char* freqBandTable,
                                   int nSfb)
{
   int est;

   for(est =  0 ; est < 2; est++){
    memcpy(tonalityDiff[est], tonalityDiff[est + noEstPerFrame], 64 * sizeof(float));
    memcpy(pSfmOrig[est], pSfmOrig[est + noEstPerFrame], 64 * sizeof(float));
    memcpy(pSfmSbr[est], pSfmSbr[est + noEstPerFrame], 64 * sizeof(float));
  }

  for(est = 0; est < noEstPerFrame; est++){
    diff(pQuotaBuffer[est+2], tonalityDiff[est+2],
         freqBandTable, nSfb, indexVector);
    calculateFlatnessMeasure(pQuotaBuffer[est+2], indexVector,
                             pSfmOrig[est+2], pSfmSbr[est+2],
                             freqBandTable, nSfb);
  }
}

static void transientCleanUp(float** quotaBuffer, float pDiffVecScfb[4][27], int nSfb,
                             unsigned char ** detectionVectors,
                             const unsigned char* pFreqBandTable,
                             GUIDE_VECTORS guideVectors,
                             int newDetectionAllowed,
                             int start, int stop)
{
  int i,j,li, ui,est;
  unsigned char pHarmVec[MAX_FREQ_COEFFS];
  for(i=0; i<MAX_FREQ_COEFFS; i++) {
	pHarmVec[i] = 0;
  }

  for(est = start; est < stop; est++){
    for(i=0;i<nSfb-1;i++){
      pHarmVec[i] = pHarmVec[i] || detectionVectors[est][i];
    }
  }

  for(i=0;i<nSfb-1;i++) {
    li = pFreqBandTable[i];
    ui = pFreqBandTable[i+1];

    if(pHarmVec[i] && pHarmVec[i+1]){
      float maxVal1, maxVal2;
      int maxPos1, maxPos2;
      li = pFreqBandTable[i];
      ui = pFreqBandTable[i+1];
      maxPos1 = li;
      maxVal1 = quotaBuffer[start][li];

      for(j = li; j<ui; j++){
        if(quotaBuffer[start][j] > maxVal1){
          maxVal1 = quotaBuffer[start][j];
          maxPos1 = j;
        }
      }

      for(est = start + 1; est < stop; est++){
        for(j = li; j<ui; j++){
          if(quotaBuffer[est][j] > maxVal1){
            maxVal1 = quotaBuffer[est][j];
            maxPos1 = j;
          }
        }
      }

      li = pFreqBandTable[i+1];
      ui = pFreqBandTable[i+2];

      maxPos2 = li;
      maxVal2 = quotaBuffer[start][li];

      for(j = li; j<ui; j++){
        if((float)quotaBuffer[start][j] > maxVal2){
          maxVal2 = quotaBuffer[start][j];
          maxPos2 = j;
        }
      }

      for(est = start + 1; est < stop; est++){
        for(j = li; j<ui; j++){
          if((float)quotaBuffer[est][j] > maxVal2){
            maxVal2 = quotaBuffer[est][j];
            maxPos2 = j;
          }
        }
      }

      if(maxPos2-maxPos1 < 2){
        if(maxVal1 > maxVal2){
          guideVectors.guideVectorDetected[i+1] = 0;
          guideVectors.guideVectorOrig[i+1] = 0;
          guideVectors.guideVectorDiff[i+1] = 0;
          for(est = start; est<stop; est++){
            detectionVectors[est][i+1] = 0;
          }
        } else{
          guideVectors.guideVectorDetected[i] = 0;
          guideVectors.guideVectorOrig[i] = 0;
          guideVectors.guideVectorDiff[i] = 0;
          for(est = start; est<stop; est++){
            detectionVectors[est][i] = 0;
          }
        }
      }
    }
  }
}

static void detection(float* quotaBuffer,
                      char* indexVector,
                      float pDiffVecScfb[27],
                      int nSfb,
                      unsigned char* pHarmVec,
                      const unsigned char* pFreqBandTable,
                      float sfmOrig[27],
                      float sfmSbr[27],
                      GUIDE_VECTORS guideVectors,
                      GUIDE_VECTORS newGuideVectors,
                      int newDetectionAllowed)
{

  int i,j,ll, lu;
  float thresTemp,thresOrig;

  for(i=0;i<nSfb;i++) {
    if (guideVectors.guideVectorDiff[i]) {
      thresTemp = max(DECAY_GUIDE_DIFF*guideVectors.guideVectorDiff[i],THR_DIFF_GUIDE);
    } else {
      thresTemp = THR_DIFF;
    }
    thresTemp = min(thresTemp, THR_DIFF);
    if(pDiffVecScfb[i] > thresTemp){
      pHarmVec[i] = 1;
      newGuideVectors.guideVectorDiff[i] = pDiffVecScfb[i];
    } else{
      if(guideVectors.guideVectorDiff[i]){
        guideVectors.guideVectorOrig[i] = THR_TONE_GUIDE ;
      }
    }
  }

  for(i=0;i<nSfb;i++){
    ll = pFreqBandTable[i];
    lu = pFreqBandTable[i+1];

    thresOrig   = max(guideVectors.guideVectorOrig[i]*DECAY_GUIDE_ORIG,THR_TONE_GUIDE );
    thresOrig   = min(thresOrig,THR_TONE);

    if(guideVectors.guideVectorOrig[i]){
      for(j= ll;j<lu;j++){
        if(quotaBuffer[j] > thresOrig){
          pHarmVec[i] = 1;
          newGuideVectors.guideVectorOrig[i] = quotaBuffer[j];
        }
      }
    }
  }

  thresOrig   = THR_TONE;

  for(i=0;i<nSfb;i++){
    ll = pFreqBandTable[i];
    lu = pFreqBandTable[i+1];

    if(lu -ll > 1){
      for(j= ll;j<lu;j++){
        if(quotaBuffer[j] > thresOrig && (sfmSbr[i] > THR_SFM_SBR && sfmOrig[i] < THR_SFM_ORIG)){
          pHarmVec[i] = 1;
          newGuideVectors.guideVectorOrig[i] = quotaBuffer[j];
        }
      }
    } else{
      if(i < nSfb -1){
        ll = pFreqBandTable[i];
        if(i>0){
          if(quotaBuffer[ll] > THR_TONE &&
             (pDiffVecScfb[+1] < I_THR_TONE ||
              pDiffVecScfb[i-1] < I_THR_TONE)) {
              pHarmVec[i] = 1;
              newGuideVectors.guideVectorOrig[i] = quotaBuffer[ll];
          }
        } else{
          if(quotaBuffer[ll] > THR_TONE &&
             pDiffVecScfb[i+1] < I_THR_TONE){
              pHarmVec[i] = 1;
              newGuideVectors.guideVectorOrig[i] = quotaBuffer[ll];
          }
        }
      }
    }
  }
}

static void detectionWithPrediction(float** quotaBuffer,
                                    char* indexVector,
                                    float pDiffVecScfb[4][27],
                                    int nSfb,
                                    const unsigned char* pFreqBandTable,
                                    float sfmOrig[4][27],
                                    float sfmSbr[4][27],
                                    unsigned char ** detectionVectors,
                                    unsigned char* prevFrameSfbHarm,
                                    GUIDE_VECTORS* guideVectors,
                                    int newDetectionAllowed,
                                    unsigned char* pAddHarmonicsScaleFactorBands)
{
  int est = 0, diff, i, k;
  int start;
  for(k=0; k<nSfb; k++) {
	pAddHarmonicsScaleFactorBands[k] = 0;
  }

  if(newDetectionAllowed){
    start = noEstPerFrame;
    memcpy(guideVectors[noEstPerFrame].guideVectorDiff,guideVectors[0].guideVectorDiff,nSfb*sizeof(float));
    memcpy(guideVectors[noEstPerFrame].guideVectorOrig,guideVectors[0].guideVectorOrig,nSfb*sizeof(float));
	for(k=0; k<32; k++) {  
      guideVectors[noEstPerFrame-1].guideVectorDetected[k] = 0;
	}
  } else{
    start = 0;
  }
  for(est = start; est < totNoEst; est++){
    if(est > 0){
      memcpy(guideVectors[est].guideVectorDetected,detectionVectors[est-1],nSfb*sizeof(unsigned char));
    }
	for(k=0; k<27; k++) {
	  detectionVectors[est][k] = 0;
	}
    if(est < totNoEst-1){
	  diff = 1;
	} else {
	  diff = 0;
	}
	for(k=0; k<32; k++) {
        guideVectors[est+diff].guideVectorDiff[k] = 0.0f;
        guideVectors[est+diff].guideVectorOrig[k] = 0.0f;
        guideVectors[est+diff].guideVectorDetected[k] = 0;
	} 
    detection(quotaBuffer[est], indexVector, pDiffVecScfb[est], nSfb,
              detectionVectors[est], pFreqBandTable, sfmOrig[est], sfmSbr[est],
              guideVectors[est], guideVectors[est+diff], newDetectionAllowed);
  }
  if(newDetectionAllowed){
    transientCleanUp(quotaBuffer,pDiffVecScfb,nSfb,detectionVectors,
					 pFreqBandTable,guideVectors[noEstPerFrame],
					 newDetectionAllowed,start,totNoEst);
  }
  for(i = 0; i< nSfb; i++){
    for(est = start; est < totNoEst; est++){
      pAddHarmonicsScaleFactorBands[i] = pAddHarmonicsScaleFactorBands[i] || detectionVectors[est][i];
    }
  }
  if(!newDetectionAllowed){
    for(i=0;i<nSfb;i++){
      if(pAddHarmonicsScaleFactorBands[i] - prevFrameSfbHarm[i] > 0) {
        pAddHarmonicsScaleFactorBands[i] = 0;
      }
    }
  }
}

void MissingHarmonicsDetectorQmf(SBR_MISSING_HARMONICS_DETECTOR *h_sbrMHDet,
                                 float ** pQuotaBuffer, char* indexVector,
							     int* pAddHarmonicsFlag, unsigned char* pAddHarmonicsScaleFactorBands,
                                 const unsigned char* freqBandTable, int nSfb)
{
  int i;
  unsigned char ** detectionVectors  = h_sbrMHDet->detectionVectors;
  GUIDE_VECTORS* guideVectors = h_sbrMHDet->guideVectors;

  calculateDetectorInput(pQuotaBuffer, indexVector, h_sbrMHDet->tonalityDiff,
                         h_sbrMHDet->sfmOrig, h_sbrMHDet->sfmSbr, freqBandTable, nSfb);

  detectionWithPrediction(pQuotaBuffer, indexVector, h_sbrMHDet->tonalityDiff, nSfb,
                          freqBandTable, h_sbrMHDet->sfmOrig, h_sbrMHDet->sfmSbr, detectionVectors,
                          h_sbrMHDet->guideScfb, guideVectors, 1,
                          pAddHarmonicsScaleFactorBands);
  *pAddHarmonicsFlag = 0;

  for(i=0;i<nSfb;i++){
    if(pAddHarmonicsScaleFactorBands[i]){
      *pAddHarmonicsFlag = 1;
      break;
    }
  }

  memcpy(h_sbrMHDet->guideScfb, pAddHarmonicsScaleFactorBands, nSfb*sizeof(unsigned char));
  memcpy(guideVectors[0].guideVectorDetected,pAddHarmonicsScaleFactorBands,nSfb*sizeof(unsigned char));

  memcpy(guideVectors[0].guideVectorDiff,guideVectors[noEstPerFrame].guideVectorDiff,nSfb*sizeof(float));
  memcpy(guideVectors[0].guideVectorOrig,guideVectors[noEstPerFrame].guideVectorOrig,nSfb*sizeof(float));

  for(i=0;i<nSfb;i++){
    if((guideVectors[0].guideVectorDiff[i] ||
        guideVectors[0].guideVectorOrig[i]) &&
      !pAddHarmonicsScaleFactorBands[i]){
      guideVectors[0].guideVectorDiff[i] = 0;
      guideVectors[0].guideVectorOrig[i] = 0;
    }
  }
}

int CreateSbrMissingHarmonicsDetector (int chan, SBR_MISSING_HARMONICS_DETECTOR *hs)
{
  int i;
  memset(hs,0,sizeof( SBR_MISSING_HARMONICS_DETECTOR));
  for(i=0; i < totNoEst; i++) {
    hs->detectionVectors[i] = &(sbr_detectionVectors[chan*NO_OF_ESTIMATES*MAX_FREQ_COEFFS + i*MAX_FREQ_COEFFS]);
  }
  return 0;
}

