/*
  Main SBR structs definitions
*/

#ifndef __SBR_H
#define __SBR_H

#include "FFR_bitbuffer.h"
#include "ton_corr.h"
#include "ffmpeg_fft.h"

typedef struct SBR_BITSTREAM_DATA
{
  int TotalBits;
  int PayloadBits;
  int FillBits;
  int HeaderActive;
  int CRCActive;
  int NrSendHeaderData;
  int CountSendHeaderData;
} SBR_BITSTREAM_DATA;

typedef struct SBR_HEADER_DATA
{
  int protocol_version;
  AMP_RES sbr_amp_res;
  int sbr_start_frequency;
  int sbr_stop_frequency;
  int sbr_xover_band;
  int sbr_noise_bands;
  int header_extra_1;
  int alterScale;
  int freqScale;

  SR_MODE sampleRateMode;

  int coupling;
  int prev_coupling;
  FFTContext fft32;
} SBR_HEADER_DATA;

typedef struct SBR_CONFIG_DATA
{
  int nChannels;

  int nSfb[2];
  int num_Master;
  int sampleFreq;

  int xOverFreq;

  unsigned char *freqBandTable[2];
  unsigned char *v_k_master;

  SBR_STEREO_MODE stereoMode;

  //int detectMissingHarmonics;
  int useParametricCoding;
} SBR_CONFIG_DATA;

typedef struct
{
  int nEnvelopes;
  int borders[MAX_ENVELOPES+1];
  int nNoiseEnvelopes;
  int bordersNoise[MAX_NOISE_ENVELOPES+1];
} SBR_FRAME_INFO;

typedef struct
{
  int upDate;
  int sfb_nrg_prev[MAX_FREQ_COEFFS];
  float dF_edge_1stEnv;
  float dF_edge_incr;
  int dF_edge_incr_fac;

  int codeBookScfLav;
  int codeBookScfLavBalance;

  int start_bits;
  int start_bits_balance;

  const int *hufftableFreqC;
  const unsigned char *hufftableFreqL;
  const int *hufftableBalanceFreqC;
  const unsigned char *hufftableBalanceFreqL;
  //const unsigned char *hufftableTimeL;
  //const unsigned char *hufftableBalanceTimeL;
} SBR_CODE_ENVELOPE;

typedef struct SBR_ENV_DATA
{
  INVF_MODE sbr_invf_mode;
  INVF_MODE sbr_invf_mode_vec[MAX_NUM_NOISE_VALUES];

  int ienvelope[MAX_ENVELOPES][MAX_FREQ_COEFFS];

  SBR_CODE_ENVELOPE EnvHuff;
  SBR_CODE_ENVELOPE NFHuff;

  //FFTContext fft32;

  int bs_num_env;
  int syntheticCoding;
  int noHarmonics;
  int addHarmonicFlag;
  unsigned char addHarmonic[MAX_FREQ_COEFFS];

  int noOfEnvelopes;
  int noScfBands[MAX_ENVELOPES];
  int domain_vec[MAX_ENVELOPES];
  int domain_vec_noise[MAX_ENVELOPES];
  int sbr_noise_levels[MAX_NUM_NOISE_VALUES];
  int noOfnoisebands;
  int balance;
  int init_sbr_amp_res;
} SBR_ENV_DATA;

typedef struct ENV_CHANNEL
{
  SBR_FRAME_INFO SbrEnvFrame;
  SBR_TON_CORR_EST   TonCorr;
  struct SBR_ENV_DATA encEnvData;
  float *qmf_states_buffer;
  float *YBuffer[QMF_TIME_SLOTS*2];
} ENV_CHANNEL;

typedef struct COMMON_DATA {
  int                   sbrDataBits;
  BIT_BUF        sbrBitbuf;
  BIT_BUF        tmpWriteBitbuf;
} COMMON_DATA;

int UpdateFreqScale(unsigned char *v_k_master, int *h_num_bands, const int k0, const int k2, const int freq_scale, const int alter_scale);
int FindStartAndStopBand(const int samplingFreq, const int startFreq, const int stop_freq, const SR_MODE sampleRateMode, int *k0, int *k2);
void Shellsort_int(int *in, int n);
void crcAdvance(unsigned short *crc, unsigned long bValue, int bBits);
void InitSbrHuffmanTables (SBR_ENV_DATA *sbrEnvData, AMP_RES amp_res);
void sbrAnalysisFiltering (FFTContext *fft32, const float *timeIn, int timeInStride, float rAnalysis[32][64], float iAnalysis[32][64], float *qmf_states_buffer);
void extractSbrEnvelope(float *timeInPtr, float *pCoreBuffer, unsigned int timeInStride,
                        struct SBR_CONFIG_DATA *h_con, struct SBR_HEADER_DATA  *sbrHeaderData,
                        struct SBR_BITSTREAM_DATA *sbrBitstreamData, struct ENV_CHANNEL *h_envChan[MAX_CHANNELS],
                        struct COMMON_DATA *cmonData);

int WriteEnvSingleChannelElement(struct SBR_ENV_DATA       *sbrEnvData,
                                 struct COMMON_DATA        *cmonData);

int WriteEnvChannelPairElement(struct SBR_ENV_DATA       *sbrEnvDataLeft,
                               struct SBR_ENV_DATA       *sbrEnvDataRight,
                               struct COMMON_DATA        *cmonData, int coupling);

#endif /* __SBR_H */

