/*
  SBR main definitions
*/
#ifndef __SBR_DEF_H
#define __SBR_DEF_H

/* Constants */
#define PI                                      3.1415926535897932f
#define EPS                                     1e-18
#define LN2                                    0.69314718056f
#define I_LN2                                   1.442695041f
#define RELAXATION                              1e-6f

/************  Definitions ***************/
#define SBR_COMP_MODE_DELTA                     0
#define SBR_COMP_MODE_CTS                       1

#define MAX_NUM_NOISE_VALUES                   10
#define MAX_NOISE_ENVELOPES                     2
#define MAX_NUM_NOISE_COEFFS                    5
#define MAX_ENVELOPES                           5
#define MAX_FREQ_COEFFS                        27
#define MAX_NUM_ENVELOPE_VALUES               (MAX_FREQ_COEFFS*MAX_ENVELOPES)
#define MAX_CHANNELS							2

#define QMF_CHANNELS                            64
#define QMF_FILTER_LENGTH                      640
#define QMF_TIME_SLOTS                          32
#define NO_OF_ESTIMATES                          4

#define NOISE_FLOOR_OFFSET                      6

#define LOW_RES                                 0
#define HIGH_RES                                1

#define LO                                      0
#define HI                                      1

#define LENGTH_SBR_FRAME_INFO                   35

#define SBR_NSFB_LOW_RES                        9
#define SBR_NSFB_HIGH_RES                       18

#define SI_SBR_PROTOCOL_VERSION_ID              0

#define SBR_FREQ_SCALE_DEFAULT                  2
#define SBR_ALTER_SCALE_DEFAULT                 1
#define SBR_NOISE_BANDS_DEFAULT                 2

#define MAX_OCTAVE        29
#define MAX_SECOND_REGION 50

/* sbr_header */
#define SI_SBR_PROTOCOL_VERSION_BITS            2
#define SI_SBR_AMP_RES_BITS                     1
#define SI_SBR_COUPLING_BITS                    1
#define SI_SBR_START_FREQ_BITS                  4
#define SI_SBR_STOP_FREQ_BITS                   4
#define SI_SBR_XOVER_BAND_BITS                  3
#define SI_SBR_RESERVED_BITS                    2
#define SI_SBR_DATA_EXTRA_BITS                  1
#define SI_SBR_HEADER_EXTRA_1_BITS              1
#define SI_SBR_HEADER_EXTRA_2_BITS              1

#define SI_SBR_FREQ_SCALE_BITS                  2
#define SI_SBR_ALTER_SCALE_BITS                 1
#define SI_SBR_NOISE_BANDS_BITS                 2

/* sbr_crc */
#define SBR_CRC_RANGE    (0x03FF)
#define SBR_CRC_MAXREGS     1
#define SBR_CRCINIT      (0x0)

#define AAC_SI_FIL_SBR             13
#define AAC_SI_FIL_SBR_CRC         14
#define SI_SBR_CRC_ENABLE_BITS                  0
#define SI_SBR_CRC_BITS                        10

#define  SI_ID_BITS_AAC             3
#define  SI_FILL_COUNT_BITS         4
#define  SI_FILL_ESC_COUNT_BITS     8
#define  SI_FILL_EXTENTION_BITS     4
#define  ID_FIL                     6

/* sbr_grid */
#define SBR_CLA_BITS                            2
#define SBR_ENV_BITS                            2

#define SBR_ABS_BITS                            2

#define SBR_NUM_BITS                            2
#define SBR_REL_BITS                            2
#define SBR_RES_BITS                            1
#define SBR_DIR_BITS                            1

/* parameters for missing harmonics detector/synthetic coding extension */
#define DELTA_TIME          9
#define MAX_COMP            2
#define TONALITY_QUOTA      0.1f
#define DIFF_QUOTA          0.75f
#define THR_DIFF           25.0f
#define THR_DIFF_GUIDE      1.26f
#define THR_TONE           15.0f
#define I_THR_TONE         (1.0f/15.0f)
#define THR_TONE_GUIDE      1.26f
#define THR_SFM_SBR         0.3f
#define THR_SFM_ORIG        0.1f
#define DECAY_GUIDE_ORIG    0.3f
#define DECAY_GUIDE_DIFF    0.5f

/* sbr_data */
#define SI_SBR_INVF_MODE_BITS                   2

#define SI_SBR_START_ENV_BITS_AMP_RES_3_0           6
#define SI_SBR_START_ENV_BITS_BALANCE_AMP_RES_3_0   5
#define SI_SBR_START_NOISE_BITS_AMP_RES_3_0         5
#define SI_SBR_START_NOISE_BITS_BALANCE_AMP_RES_3_0 5
#define SI_SBR_START_ENV_BITS_AMP_RES_1_5           7
#define SI_SBR_START_ENV_BITS_BALANCE_AMP_RES_1_5   6

#define SI_SBR_EXTENDED_DATA_BITS               1
#define SBR_EXTENDED_DATA_MAX_CNT               (15+255)

/* Envelope coding constants */
#define FREQ                      0
#define TIME                      1

/* huffman tables */
#define CODE_BOOK_SCF_LAV00         60
#define CODE_BOOK_SCF_LAV01         31
#define CODE_BOOK_SCF_LAV10         60
#define CODE_BOOK_SCF_LAV11         31
#define CODE_BOOK_SCF_LAV_BALANCE11 12
#define CODE_BOOK_SCF_LAV_BALANCE10 24

typedef enum
{
  SBR_AMP_RES_1_5=0,
  SBR_AMP_RES_3_0
} AMP_RES;

typedef enum
{
  INVF_OFF = 0,
  INVF_LOW_LEVEL,
  INVF_MID_LEVEL,
  INVF_HIGH_LEVEL,
  INVF_SWITCHED
} INVF_MODE;

typedef enum
{
  SINGLE_RATE,
  DUAL_RATE
} SR_MODE;

typedef enum
{
  FREQ_RES_LOW,
  FREQ_RES_HIGH
} FREQ_RES;

typedef enum
{
  SBR_MONO,
  SBR_LEFT_RIGHT,
  SBR_COUPLING,
} SBR_STEREO_MODE;

#ifndef min
#define min(a, b) ((a) < (b) ? (a) : (b))
#endif

#ifndef max
#define max(a, b) ((a) > (b) ? (a) : (b))
#endif

#endif
