/*
  SBR encoder top level processing
*/
#include <stdlib.h>
#include <string.h>
#include "sbr_main.h"
#include "sbr.h"

#define INVALID_TABLE_IDX -1

unsigned char    sbr_freqBandTableLO[MAX_FREQ_COEFFS/2+1];
unsigned char    sbr_freqBandTableHI[MAX_FREQ_COEFFS+1];
unsigned char    sbr_v_k_master[MAX_FREQ_COEFFS +1];

/*
   tuningTable
*/
const struct
{
  unsigned int    bitrateFrom;
  unsigned int    bitrateTo;

  unsigned int    sampleRate;
  unsigned int    numChannels;

  unsigned int    startFreq;
  unsigned int    stopFreq;

  int numNoiseBands;
  //int noiseFloorOffset;
  int noiseMaxLevel;
  SBR_STEREO_MODE stereoMode;
  int freqScale;

} tuningTable[] = {


  /*** mono ***/
  { 10000, 24001,  11025, 1,  1,  3, 1, 6, SBR_MONO, 3 }, /* lol */

  { 10000, 12000,  16000, 1,  1,  3, 1, 6, SBR_MONO, 3 }, /* nominal: 10 kbit/s */
  { 12000, 16000,  16000, 1,  2,  0, 1, 6, SBR_MONO, 3 }, /* nominal: 12 kbit/s */
  { 16000, 20000,  16000, 1,  2,  3, 1, 6, SBR_MONO, 3 }, /* nominal: 18 kbit/s */

  { 12000, 18000,  22050, 1,  3,  5, 1, -3, SBR_MONO, 3 }, /* nominal: 14 kbit/s */
  { 18000, 22000,  22050, 1,  3,  5, 2, -3, SBR_MONO, 2 }, /* nominal: 20 kbit/s */
  { 22000, 28000,  22050, 1,  7,  8, 2, -3, SBR_MONO, 2 }, /* nominal: 24 kbit/s */
  { 28000, 36000,  22050, 1, 11,  9, 2, -3, SBR_MONO, 2 }, /* nominal: 32 kbit/s */
  { 36000, 44001,  22050, 1, 11,  9, 2, -3, SBR_MONO, 1 }, /* nominal: 40 kbit/s */
  { 44002, 64001,  22050, 1, 12, 10, 2, -3, SBR_MONO, 1 }, /* nominal: 48 kbit/s */

  { 12000, 18000,  24000, 1,  1,  4, 1, 6, SBR_MONO, 3 }, /* nominal: 14 kbit/s */
  { 18000, 22000,  24000, 1,  3,  5, 2, 6, SBR_MONO, 2 }, /* nominal: 20 kbit/s */
  { 22000, 28000,  24000, 1,  7,  8, 2, 6, SBR_MONO, 2 }, /* nominal: 24 kbit/s */
  { 28000, 36000,  24000, 1, 10,  9, 2, 3, SBR_MONO, 2 }, /* nominal: 32 kbit/s */
  { 36000, 44001,  24000, 1, 11,  9, 2, 3, SBR_MONO, 1 }, /* nominal: 40 kbit/s */

  /*** stereo ***/
  { 18000, 24000,  16000, 2,  4,  2, 1, -3, SBR_COUPLING, 3 }, /* nominal: 18 kbit/s */
  
  { 16000, 28000,  22050, 2,  7,  8, 1, -3, SBR_COUPLING, 3 }, /* nominal: 24 kbit/s */
  { 28000, 36000,  22050, 2, 10,  9, 1, -3, SBR_COUPLING, 2 }, /* nominal: 32 kbit/s */
#ifdef __LP64__
  { 36000, 44000,  22050, 2, 10,  9, 2, -3, SBR_COUPLING, 2 }, /* nominal: 40 kbit/s */
#else
  { 36000, 44000,  22050, 2, 12,  10, 2, -3, SBR_COUPLING, 2 }, /* nominal: 40 kbit/s */
#endif

  { 44000, 80001,  22050, 2, 12, 10, 3, -3, SBR_COUPLING, 2 }, /* nominal: 64 kbit/s */
  { 80002, 128001, 22050, 2, 14, 10, 3, -3, SBR_COUPLING, 2 }, 

  { 24000, 28000,  24000, 2,  5,  6, 1, -3, SBR_COUPLING, 3 }, /* nominal: 24 kbit/s */
  { 28000, 36000,  24000, 2,  7,  8, 2, -3, SBR_COUPLING, 2 }, /* nominal: 32 kbit/s */
  { 36000, 44000,  24000, 2, 10,  9, 2, -3, SBR_COUPLING, 2 }, /* nominal: 40 kbit/s */
  { 44000, 64000,  24000, 2, 12,  9, 3, -3, SBR_COUPLING, 2 }, /* nominal: 48 kbit/s */

};

struct ENV_CHANNEL EnvChannel[MAX_CHANNELS];
struct SBR_ENCODER sbrEncoder;
/***************************************************************************
  \brief  Selects the SBR tuning.
****************************************************************************/
static int getSbrTuningTableIndex(unsigned int bitrate, unsigned int numChannels, unsigned int sampleRate)
{
  int i, paramSets = sizeof (tuningTable) / sizeof (tuningTable [0]);
  for (i = 0 ; i < paramSets ; i++)  {
    if (numChannels == tuningTable [i].numChannels) {
      if ((sampleRate == tuningTable [i].sampleRate) &&
          (bitrate >= tuningTable [i].bitrateFrom) &&
          (bitrate < tuningTable [i].bitrateTo)) {
        return i ;
      }
    }
  }
  return INVALID_TABLE_IDX;
}

/***************************************************************************/
/*!

  \brief  Is SBR setting available.
  \return a flag indicating success: yes (1) or no (0)

****************************************************************************/
unsigned int IsSbrSettingAvail (unsigned int bitrate, unsigned int numOutputChannels,
				unsigned int sampleRateInput, unsigned int *sampleRateCore)
{
  int idx = INVALID_TABLE_IDX;
  /* if (sampleRateInput < 32000) {
    return 0;
  } */
  *sampleRateCore = sampleRateInput/2;
  idx = getSbrTuningTableIndex(bitrate,numOutputChannels, *sampleRateCore);
  return (idx == INVALID_TABLE_IDX) ? 0 : 1;
}


/*****************************************************************************

 description:  initializes the SBR confifuration
 returns:      error status

*****************************************************************************/
static inline void EnvInitDefaultSettings (sbrConfigurationPtr config)
{
  config->SendHeaderDataTime     = 500;
  config->crcSbr                 = 1;
  config->tran_thr               = 13000;
  config->parametricCoding       = 1;
  config->useSpeechConfig        = 0;
  //config->amp_res                = SBR_AMP_RES_3_0 ;
  config->amp_res                = SBR_AMP_RES_1_5;
  config->tran_fc                = 0 ;
  config->tran_det_mode          = 1 ;
  config->e                      = 1 ;
  config->dF_edge_1stEnv         = 0.3f ;
  config->dF_edge_incr           = 0.3f ;
  config->sbr_invf_mode   = INVF_SWITCHED;
  config->stereoMode             = SBR_COUPLING;
  config->ana_max_level          = 6;
  config->startFreq              = 5;
  config->stopFreq               = 9;
  config->freqScale       = SBR_FREQ_SCALE_DEFAULT;
  config->alterScale      = SBR_ALTER_SCALE_DEFAULT;
  config->sbr_noise_bands = SBR_NOISE_BANDS_DEFAULT;
}

/***************************************************************************/
/*!

  \brief  Adjusts the SBR settings.
  \return success:

****************************************************************************/
unsigned int EnvAdjustSettings (const sbrConfigurationPtr config, unsigned int bitRate, unsigned int numChannels,
				unsigned int fsCore, unsigned int transFac, unsigned int standardBitrate)
{
  int idx = INVALID_TABLE_IDX;
  //unsigned int sampleRate;

  /* set the codec settings */
  EnvInitDefaultSettings(config);
  config->codecSettings.bitRate         = bitRate;
  config->codecSettings.nChannels       = numChannels;
  config->codecSettings.sampleFreq      = fsCore;
  config->codecSettings.transFac        = transFac;
  config->codecSettings.standardBitrate = standardBitrate;

  //sampleRate  = fsCore * 2;
  idx = getSbrTuningTableIndex(bitRate,numChannels,fsCore);

  if (idx != INVALID_TABLE_IDX) {
    config->startFreq       = tuningTable [idx].startFreq ;
    config->stopFreq        = tuningTable [idx].stopFreq ;
    config->sbr_noise_bands = tuningTable [idx].numNoiseBands ;
    config->ana_max_level   = tuningTable [idx].noiseMaxLevel ;
    config->stereoMode      = tuningTable[idx].stereoMode ;
    config->freqScale       = tuningTable[idx].freqScale ;
    if (bitRate <= 12000) {
      config->parametricCoding = 0;
      config->useSpeechConfig  = 1;
    }
    return 1;
  }

  return 0;
}

/*****************************************************************************
 description:  close the envelope coding handle
*****************************************************************************/
void EnvClose (SBR_ENCODER *hEnvEnc)
{
  int i;

  if (hEnvEnc != NULL) {
    for (i = 0; i < MAX_CHANNELS; i++) {
      if (hEnvEnc->hEnvChannel[i] != NULL) {
        hEnvEnc->hEnvChannel[i] = NULL;
      }
    }
  }
}

static void UpdateHiRes(unsigned char *h_hires, int *num_hires,unsigned char * v_k_master,
                int num_master, int *xover_band, SR_MODE drOrSr)
{
  int i;
  int divider;
  int max1,max2;
  divider = (drOrSr == DUAL_RATE) ? 2 : 1;

  if((v_k_master[*xover_band] > (QMF_CHANNELS/divider)) || (*xover_band > num_master)) {
    max1=0;
    max2=num_master;
    while((v_k_master[max1+1] < (QMF_CHANNELS/divider)) && ((max1+1) < max2)) {
        max1++;
    }
    *xover_band=max1;
  }

  *num_hires = num_master - *xover_band;

  for(i = *xover_band; i <= num_master; i++) {
      h_hires[i - *xover_band] = v_k_master[i];
  }
}

static void UpdateLoRes(unsigned char * h_lores, int *num_lores, unsigned char * h_hires, int num_hires)
{
  int i;
  if(num_hires%2 == 0) {
      *num_lores=num_hires/2;
      for(i=0;i<=*num_lores;i++) {
        h_lores[i]=h_hires[i*2];
      }
  } else {
      *num_lores=(num_hires+1)/2;
      h_lores[0]=h_hires[0];
      for(i=1;i<=*num_lores;i++) {
          h_lores[i]=h_hires[i*2-1];
      }
  }
}

/*****************************************************************************
 description:  updates vk_master
*****************************************************************************/
int updateFreqBandTable(SBR_CONFIG_DATA *sbrConfigData, SBR_HEADER_DATA *sbrHeaderData)
{
  int k0, k2;
  if(FindStartAndStopBand(sbrConfigData->sampleFreq,
                          sbrHeaderData->sbr_start_frequency,
                          sbrHeaderData->sbr_stop_frequency,
                          sbrHeaderData->sampleRateMode,
                          &k0, &k2))
  {
    return(1);
  }

  if(UpdateFreqScale(sbrConfigData->v_k_master, &sbrConfigData->num_Master,
                     k0, k2, sbrHeaderData->freqScale,
                     sbrHeaderData->alterScale))
  {
    return(1);
  }

  sbrHeaderData->sbr_xover_band=0;
  UpdateHiRes(sbrConfigData->freqBandTable[HI],
                 &sbrConfigData->nSfb[HI],
                 sbrConfigData->v_k_master,
                 sbrConfigData->num_Master ,
                 &sbrHeaderData->sbr_xover_band,
                 sbrHeaderData->sampleRateMode);
  UpdateLoRes(sbrConfigData->freqBandTable[LO],
              &sbrConfigData->nSfb[LO],
              sbrConfigData->freqBandTable[HI],
              sbrConfigData->nSfb[HI]);

  sbrConfigData->xOverFreq = (sbrConfigData->freqBandTable[LOW_RES][0] * sbrConfigData->sampleFreq / QMF_CHANNELS+1)>>1;
  return (0);
}

static inline void InitSbrBitstream(COMMON_DATA *hCmonData, unsigned char *memoryBase, int memorySize, int CRCActive)
{
  ResetBitBuf(&hCmonData->sbrBitbuf,memoryBase,memorySize);
  hCmonData->tmpWriteBitbuf = hCmonData->sbrBitbuf;
  WriteBits (&hCmonData->sbrBitbuf, 0, SI_FILL_EXTENTION_BITS);
  if(CRCActive) {
    WriteBits (&hCmonData->sbrBitbuf, 0,SI_SBR_CRC_BITS);
  }
}

static inline void AssembleSbrBitstream(COMMON_DATA *hCmonData, int sbrHdrBits, int CRCActive)
{
  unsigned short crcReg =  SBR_CRCINIT;
  int numCrcBits, i, sbrFillBits;
  int sbrLoad=0;

  if (hCmonData==NULL) {
    return;
  }

  sbrLoad = sbrHdrBits + hCmonData->sbrDataBits;
  sbrLoad += SI_FILL_EXTENTION_BITS;

  if (CRCActive) {
    sbrLoad += SI_SBR_CRC_BITS;
  }

  sbrFillBits = (8 - (sbrLoad) % 8) % 8;
  sbrLoad += sbrFillBits;
  WriteBits(&hCmonData->sbrBitbuf, 0, sbrFillBits);
  if (CRCActive){
      unsigned char *bufptr = hCmonData->sbrBitbuf.pBitBufBase;
      bufptr++;
      unsigned int bit2;
      numCrcBits = sbrHdrBits + hCmonData->sbrDataBits + sbrFillBits;
      bit2 = *bufptr++;
      bit2 &= 3;
      crcAdvance(&crcReg, bit2, 2);
      numCrcBits -= 2;
      numCrcBits /= 8;
      for(i=0;i<numCrcBits;i++){
        unsigned int bit = *bufptr++;
        crcAdvance(&crcReg, bit, 8);
      }
    crcReg &= (SBR_CRC_RANGE);
  }

  if (CRCActive) {
    WriteBits (&hCmonData->tmpWriteBitbuf, AAC_SI_FIL_SBR_CRC, SI_FILL_EXTENTION_BITS);
    WriteBits (&hCmonData->tmpWriteBitbuf, crcReg,SI_SBR_CRC_BITS);
  } else {
    WriteBits (&hCmonData->tmpWriteBitbuf, AAC_SI_FIL_SBR, SI_FILL_EXTENTION_BITS);
  }
}

/*****************************************************************************
    description:  writes the SBR header
*****************************************************************************/
static inline int EncodeSbrHeaderData (SBR_HEADER_DATA *sbrHeaderData, BIT_BUF *hBitStream)
{
  int payloadBits = 0;

  if (sbrHeaderData != NULL) {
    payloadBits += WriteBits (hBitStream, sbrHeaderData->sbr_amp_res, SI_SBR_AMP_RES_BITS);
    payloadBits += WriteBits (hBitStream, sbrHeaderData->sbr_start_frequency, SI_SBR_START_FREQ_BITS);
    payloadBits += WriteBits (hBitStream, sbrHeaderData->sbr_stop_frequency, SI_SBR_STOP_FREQ_BITS);
    payloadBits += WriteBits (hBitStream, sbrHeaderData->sbr_xover_band, SI_SBR_XOVER_BAND_BITS);
    payloadBits += WriteBits (hBitStream, 0, SI_SBR_RESERVED_BITS);
    payloadBits += WriteBits (hBitStream, sbrHeaderData->header_extra_1, SI_SBR_HEADER_EXTRA_1_BITS);
    payloadBits += WriteBits (hBitStream, 0, SI_SBR_HEADER_EXTRA_2_BITS);
    if (sbrHeaderData->header_extra_1) {
      payloadBits += WriteBits (hBitStream, sbrHeaderData->freqScale, SI_SBR_FREQ_SCALE_BITS);
      payloadBits += WriteBits (hBitStream, sbrHeaderData->alterScale, SI_SBR_ALTER_SCALE_BITS);
      payloadBits += WriteBits (hBitStream, sbrHeaderData->sbr_noise_bands, SI_SBR_NOISE_BANDS_BITS);
    } /* sbrHeaderData->header_extra_1 */
  } /* sbrHeaderData != NULL */
  return payloadBits;
}

/*****************************************************************************

 description: performs the sbr envelope calculation
 returns:

*****************************************************************************/
int
EnvEncodeFrame (SBR_ENCODER *hEnvEncoder,
                float *samples,
                float *pCoreBuffer,
                unsigned int timeInStride,
                unsigned int  *numAncBytes,
                unsigned char *ancData)
{
  int sbrHdrBits = 0;
  if (hEnvEncoder != NULL) {
    /* header bitstream handling */
    SBR_BITSTREAM_DATA *sbrBitstreamData = &hEnvEncoder->sbrBitstreamData;
    sbrBitstreamData->HeaderActive = 0;
    if (sbrBitstreamData->CountSendHeaderData == 0) {
      sbrBitstreamData->HeaderActive = 1;
    }
    if (sbrBitstreamData->NrSendHeaderData == 0) {
      sbrBitstreamData->CountSendHeaderData = 1;
    } else {
      sbrBitstreamData->CountSendHeaderData++;
      sbrBitstreamData->CountSendHeaderData %= sbrBitstreamData->NrSendHeaderData;
    }

    InitSbrBitstream(&hEnvEncoder->CmonData,
                     (unsigned char*) hEnvEncoder->sbrPayload,
                     sizeof(hEnvEncoder->sbrPayload),
                     hEnvEncoder->sbrBitstreamData.CRCActive);

    sbrHdrBits = 1; 
    if (sbrBitstreamData->HeaderActive) {
      WriteBits (&hEnvEncoder->CmonData.sbrBitbuf, 1, 1);
      sbrHdrBits += EncodeSbrHeaderData(&hEnvEncoder->sbrHeaderData, &hEnvEncoder->CmonData.sbrBitbuf);
    } else {
      WriteBits (&hEnvEncoder->CmonData.sbrBitbuf, 0, 1);
    }

    extractSbrEnvelope(samples, pCoreBuffer, timeInStride,
                       &hEnvEncoder->sbrConfigData,
                       &hEnvEncoder->sbrHeaderData,
                       &hEnvEncoder->sbrBitstreamData,
                       hEnvEncoder->hEnvChannel,
                       &hEnvEncoder->CmonData);

    AssembleSbrBitstream(&hEnvEncoder->CmonData, sbrHdrBits, hEnvEncoder->sbrBitstreamData.CRCActive);
    hEnvEncoder->sbrPayloadSize = GetBitsAvail(&hEnvEncoder->CmonData.sbrBitbuf)/8;
    if(hEnvEncoder->sbrPayloadSize > MAX_PAYLOAD_SIZE) {
      hEnvEncoder->sbrPayloadSize=0;
    }

   if(ancData){
      *numAncBytes = hEnvEncoder->sbrPayloadSize;
      memcpy(ancData,hEnvEncoder->sbrPayload,hEnvEncoder->sbrPayloadSize);
    }
  }
 return (0);
}

extern float __attribute__ ((aligned (16))) sbr_QmfStatesAnalysis[2*QMF_FILTER_LENGTH];
extern float sbr_envYBuffer[MAX_CHANNELS  * QMF_TIME_SLOTS * QMF_CHANNELS];
/*****************************************************************************

 description:  initializes parameters and allocates memory
 returns:      error status

*****************************************************************************/

int
createEnvChannel (int chan,
                  SBR_CONFIG_DATA *sbrConfigData,
                  SBR_HEADER_DATA *sbrHeaderData,
                  ENV_CHANNEL     *hEnv,
                  sbrConfigurationPtr params)
{

  int i, e;

  e = 1 << params->e;
  if(CreateTonCorrParamExtr (chan,
                             &hEnv->TonCorr,



                             sbrConfigData->sampleFreq,
                             sbrConfigData->freqBandTable[LOW_RES][sbrConfigData->nSfb[LO]],
                             sbrConfigData->freqBandTable[LOW_RES][0],
                             0,
                             sbrConfigData->v_k_master,
                             sbrConfigData->num_Master,
                             params->ana_max_level,
                             sbrConfigData->freqBandTable,
                             sbrConfigData->nSfb,
                             sbrHeaderData->sbr_noise_bands,
                             params->useSpeechConfig))
  {
    return(1);
  }

  hEnv->encEnvData.noOfnoisebands = hEnv->TonCorr.sbrNoiseFloorEstimate.noNoiseBands;
  hEnv->encEnvData.sbr_invf_mode = (INVF_MODE)params->sbr_invf_mode;
  if (hEnv->encEnvData.sbr_invf_mode == INVF_SWITCHED) {
    hEnv->encEnvData.sbr_invf_mode = INVF_MID_LEVEL;
  }

  hEnv->qmf_states_buffer = sbr_QmfStatesAnalysis + chan * 640;
  for (i = 0; i < 32; i++) {
    hEnv->YBuffer[i] = &sbr_envYBuffer[chan*QMF_TIME_SLOTS*64 + i*QMF_CHANNELS];
  }

  ff_fft_init(&sbrHeaderData->fft32, 5, 1);
  hEnv->encEnvData.EnvHuff.dF_edge_1stEnv = params->dF_edge_1stEnv;
  hEnv->encEnvData.EnvHuff.dF_edge_incr = params->dF_edge_incr;
  hEnv->encEnvData.EnvHuff.upDate = 0;
  hEnv->encEnvData.NFHuff.dF_edge_1stEnv = 0.0f;
  hEnv->encEnvData.NFHuff.dF_edge_incr = 0.0f;
  hEnv->encEnvData.NFHuff.upDate = 0;

  InitSbrHuffmanTables(&hEnv->encEnvData, sbrHeaderData->sbr_amp_res);
  hEnv->encEnvData.bs_num_env = e;
  hEnv->encEnvData.noHarmonics = sbrConfigData->nSfb[HI];
  hEnv->encEnvData.addHarmonicFlag = 0;
  return (0);
}


/*****************************************************************************
 description:  initializes parameters and allocates memory
*****************************************************************************/
int EnvOpen (SBR_ENCODER **hEnvEncoder, float *pCoreBuffer, sbrConfigurationPtr params, int *coreBandWith)
{
  SBR_ENCODER *hEnvEnc;
  int ch;
  *hEnvEncoder=0;
  hEnvEnc = &sbrEncoder;

  for (ch=0; ch<MAX_CHANNELS; ch++) {
    hEnvEnc->hEnvChannel[ch] = &EnvChannel[ch];
  }

  if ((params->codecSettings.nChannels < 1) || (params->codecSettings.nChannels > MAX_CHANNELS)){
    EnvClose (hEnvEnc);
    return(1);
  }

  hEnvEnc->sbrConfigData.freqBandTable[LO] =  sbr_freqBandTableLO;
  memset(hEnvEnc->sbrConfigData.freqBandTable[LO],0,sizeof(unsigned char)*MAX_FREQ_COEFFS/2+1);
  hEnvEnc->sbrConfigData.freqBandTable[HI] =  sbr_freqBandTableHI;
  memset(hEnvEnc->sbrConfigData.freqBandTable[HI],0,sizeof(unsigned char)*MAX_FREQ_COEFFS+1);
  hEnvEnc->sbrConfigData.v_k_master =  sbr_v_k_master;
  memset(hEnvEnc->sbrConfigData.v_k_master,0,sizeof(unsigned char)*MAX_FREQ_COEFFS+1);

  CreateBitBuffer(&hEnvEnc->CmonData.sbrBitbuf,
                  (unsigned char*) hEnvEnc->sbrPayload,
                  sizeof(hEnvEnc->sbrPayload));

  hEnvEnc->sbrConfigData.nChannels = params->codecSettings.nChannels;

  if(params->codecSettings.nChannels == 2) {
     hEnvEnc->sbrConfigData.stereoMode = params->stereoMode;
  } else {
     hEnvEnc->sbrConfigData.stereoMode = SBR_MONO;
  }

  if (params->codecSettings.sampleFreq <= 24000) {
    hEnvEnc->sbrHeaderData.sampleRateMode = DUAL_RATE;
    hEnvEnc->sbrConfigData.sampleFreq = 2 * params->codecSettings.sampleFreq;
  } else {
    hEnvEnc->sbrHeaderData.sampleRateMode = SINGLE_RATE;
    hEnvEnc->sbrConfigData.sampleFreq = params->codecSettings.sampleFreq;
  }

  hEnvEnc->sbrBitstreamData.CountSendHeaderData = 0;
  if (params->SendHeaderDataTime > 0 ) {
    hEnvEnc->sbrBitstreamData.NrSendHeaderData = (int)(params->SendHeaderDataTime * 0.001*
                                                       hEnvEnc->sbrConfigData.sampleFreq / 2048);
    hEnvEnc->sbrBitstreamData.NrSendHeaderData = max(hEnvEnc->sbrBitstreamData.NrSendHeaderData,1);
  } else {
   hEnvEnc->sbrBitstreamData.NrSendHeaderData = 0;
  }

  hEnvEnc->sbrBitstreamData.CRCActive = params->crcSbr;
  hEnvEnc->sbrBitstreamData.HeaderActive = 0;
  hEnvEnc->sbrHeaderData.sbr_start_frequency = params->startFreq;
  hEnvEnc->sbrHeaderData.sbr_stop_frequency  = params->stopFreq;
  hEnvEnc->sbrHeaderData.sbr_xover_band = 0;

   hEnvEnc->sbrHeaderData.protocol_version = SI_SBR_PROTOCOL_VERSION_ID;
   hEnvEnc->sbrHeaderData.sbr_amp_res = (AMP_RES)params->amp_res;

  hEnvEnc->sbrHeaderData.freqScale  = params->freqScale;
  hEnvEnc->sbrHeaderData.alterScale = params->alterScale;
  hEnvEnc->sbrHeaderData.sbr_noise_bands = params->sbr_noise_bands;
  hEnvEnc->sbrHeaderData.header_extra_1 = 0;

  if ((params->freqScale != SBR_FREQ_SCALE_DEFAULT) ||
      (params->alterScale != SBR_ALTER_SCALE_DEFAULT) ||
      (params->sbr_noise_bands != SBR_NOISE_BANDS_DEFAULT)) {
   hEnvEnc->sbrHeaderData.header_extra_1 = 1;
  }

  hEnvEnc->sbrConfigData.useParametricCoding       = params->parametricCoding;
  if(updateFreqBandTable(&hEnvEnc->sbrConfigData, &hEnvEnc->sbrHeaderData)){
    EnvClose (hEnvEnc);
    return(1);
  }

  for ( ch = 0; ch < hEnvEnc->sbrConfigData.nChannels; ch++ ) {
    if(createEnvChannel(ch, &hEnvEnc->sbrConfigData, &hEnvEnc->sbrHeaderData, hEnvEnc->hEnvChannel[ch], params)){
      EnvClose (hEnvEnc);
      return(1);
    }
  }

  hEnvEnc->sbrPayloadSize = 0;
  *hEnvEncoder = hEnvEnc;
  *coreBandWith = hEnvEnc->sbrConfigData.xOverFreq;
  return 0;
}

