/*
  SBR encoder top level processing prototypes
*/

#ifndef __SBR_MAIN_H
#define __SBR_MAIN_H

#include "sbr.h"

#define MAX_TRANS_FAC         8
#define MAX_CODEC_FRAME_RATIO 2
#define MAX_PAYLOAD_SIZE    256

typedef struct
{
  int bitRate;
  int nChannels;
  int sampleFreq;
  int transFac;
  int standardBitrate;
} CODEC_PARAM;

typedef struct sbrConfiguration
{
  CODEC_PARAM codecSettings;
  int SendHeaderDataTime;
  int crcSbr;
  //int detectMissingHarmonics;
  int parametricCoding;

  int tran_thr;
  unsigned int useSpeechConfig;

  int amp_res;
  int ana_max_level;
  int tran_fc;
  int tran_det_mode;
  int e;
  SBR_STEREO_MODE stereoMode;
  float dF_edge_1stEnv;
  float dF_edge_incr;
  int sbr_invf_mode;
  int startFreq;
  int stopFreq;

  int freqScale;
  int alterScale;
  int sbr_noise_bands;
} sbrConfiguration, *sbrConfigurationPtr;

typedef struct SBR_ENCODER {
  struct SBR_CONFIG_DATA    sbrConfigData;
  struct SBR_HEADER_DATA    sbrHeaderData;
  struct SBR_BITSTREAM_DATA sbrBitstreamData;
  struct ENV_CHANNEL*       hEnvChannel[2];
  struct COMMON_DATA        CmonData;
  unsigned int              sbrPayloadPrevious[MAX_PAYLOAD_SIZE/(sizeof(int))];
  unsigned int              sbrPayload[MAX_PAYLOAD_SIZE/(sizeof(int))];
  int                       sbrPayloadSize;
} SBR_ENCODER;

unsigned int IsSbrSettingAvail (unsigned int bitrate, unsigned int numOutputChannels,
                                unsigned int sampleRateInput, unsigned int *sampleRateCore);

unsigned int EnvAdjustSettings (const sbrConfigurationPtr config, unsigned int bitRate, unsigned int numChannels,
				unsigned int fsCore, unsigned int transFac, unsigned int standardBitrate);

int EnvOpen (SBR_ENCODER **hEnvEncoder, float *pCoreBuffer, sbrConfigurationPtr params, int *coreBandWith);
void EnvClose (SBR_ENCODER *hEnvEnc);

int EnvEncodeFrame (SBR_ENCODER *hEnvEncoder, float *samples, float *pCoreBuffer,
		    unsigned int  timeInStride, unsigned int  *numAncBytes, unsigned char *ancData);

#endif /* ifndef __SBR_MAIN_H */
