/*
  General tonality correction detector module.
*/
#include <string.h>
#include <math.h>
#include "ton_corr.h"
#define abs(x) ((x >= 0) ? x : -x)


float sbr_quotaMatrix[512]; //2*NO_OF_ESTIMATES*QMF_CHANNELS
/**************************************************************************
  \brief Extracts the parameters required in the decoder.
**************************************************************************/
void TonCorrParamExtr(SBR_TON_CORR_EST *hTonCorr, INVF_MODE* infVec, float* noiseLevels,
					  int* missingHarmonicFlag, unsigned char * missingHarmonicsIndex,
                      int nNoiseEnvelopes, unsigned char* freqBandTable, int nSfb)
{
  int band;
  INVF_MODE* infVecPtr;

  InverseFilteringDetectorQmf(&hTonCorr->sbrInvFilt, hTonCorr->quotaMatrix, hTonCorr->nrgVector,
                               hTonCorr->indexVector, 0, infVec);
  MissingHarmonicsDetectorQmf(&hTonCorr->sbrMissingHarmonicsDetector,
                              hTonCorr->quotaMatrix, hTonCorr->indexVector,
                              missingHarmonicFlag, missingHarmonicsIndex,
                              freqBandTable, nSfb);
  infVecPtr = hTonCorr->sbrInvFilt.prevInvfMode;
  NoiseFloorEstimateQmf(&hTonCorr->sbrNoiseFloorEstimate, nNoiseEnvelopes,
                        noiseLevels, hTonCorr->quotaMatrix, hTonCorr->indexVector,
                        infVecPtr);

  for(band = 0 ; band < hTonCorr->sbrInvFilt.noDetectorBands; band++){
    hTonCorr->sbrInvFilt.prevInvfMode[band] = infVec[band];
  }
}

static int findClosestEntry(int goalSb, unsigned char *v_k_master, int numMaster, int direction)
{
  int index;

  if(goalSb <= v_k_master[0]) {
    return v_k_master[0];
  }

  if(goalSb >= v_k_master[numMaster]) {
    return v_k_master[numMaster];
  }

  if(direction) {
    index = 0;
    while(v_k_master[index] < goalSb) {
      index++;
    }
  } else {
    index = numMaster;
    while(v_k_master[index] > goalSb) {
      index--;
    }
  }

  return v_k_master[index];
}

static int resetPatch(SBR_TON_CORR_EST *hTonCorr, int highBandStartSb, int channelOffset, unsigned char *v_k_master, int numMaster, int fs)
{
  int patch,k,i;
  int targetStopBand;

  PATCH_PARAM  *patchParam = hTonCorr->patchParam;

  int sbGuard = hTonCorr->guard;
  int sourceStartBand;
  int patchDistance;
  int numBandsInPatch;

  int lsb = v_k_master[0];
  int usb = v_k_master[numMaster];
  int xoverOffset = highBandStartSb - v_k_master[0];

  int goalSb;

  goalSb = (int)(2.0f * 64.0f * 16000.0f / fs  + 0.5f);
  goalSb = findClosestEntry(goalSb, v_k_master, numMaster, 1);
  sourceStartBand = hTonCorr->shiftStartSb + xoverOffset;
  targetStopBand = lsb + xoverOffset;
  patch = 0;

  while(targetStopBand < usb) {
    if (patch >= MAX_NUM_PATCHES) {
      return(1);
    }

    patchParam[patch].guardStartBand = targetStopBand;
    targetStopBand += sbGuard;
    patchParam[patch].targetStartBand = targetStopBand;
    numBandsInPatch = goalSb - targetStopBand;

    if (numBandsInPatch >= lsb - sourceStartBand) {
      patchDistance   = targetStopBand - sourceStartBand;
      patchDistance   = patchDistance & ~1;
      numBandsInPatch = lsb - (targetStopBand - patchDistance);
      numBandsInPatch = findClosestEntry(targetStopBand + numBandsInPatch, v_k_master, numMaster, 0) -
                        targetStopBand;
    }

    patchDistance   = numBandsInPatch + targetStopBand - lsb;
    patchDistance   = (patchDistance + 1) & ~1;

    if (numBandsInPatch <= 0) {
      patch--;
    } else {
      patchParam[patch].sourceStartBand = targetStopBand - patchDistance;
      patchParam[patch].targetBandOffs  = patchDistance;
      patchParam[patch].numBandsInPatch = numBandsInPatch;
      patchParam[patch].sourceStopBand  = patchParam[patch].sourceStartBand + numBandsInPatch;
      targetStopBand += patchParam[patch].numBandsInPatch;
    }
    sourceStartBand = hTonCorr->shiftStartSb;
    if(abs(targetStopBand - goalSb) < 3) {
      goalSb = usb;
    }
    patch++;
  }
  patch--;

  if (patchParam[patch].numBandsInPatch < 3 && patch > 0) {
    patch--;
    targetStopBand = patchParam[patch].targetStartBand + patchParam[patch].numBandsInPatch;
  }

  hTonCorr->noOfPatches = patch + 1;

  for(k = 0; k < hTonCorr->patchParam[0].guardStartBand; k++) {
    hTonCorr->indexVector[k] = k;
  }

  for(i = 0; i < hTonCorr->noOfPatches; i++) {
    int sourceStart    = hTonCorr->patchParam[i].sourceStartBand;
    int targetStart    = hTonCorr->patchParam[i].targetStartBand;
    int numberOfBands  = hTonCorr->patchParam[i].numBandsInPatch;
    int startGuardBand = hTonCorr->patchParam[i].guardStartBand;

    for(k = 0; k < (targetStart- startGuardBand); k++) {
      hTonCorr->indexVector[startGuardBand+k] = -1;
    }

    for(k = 0; k < numberOfBands; k++) {
      hTonCorr->indexVector[targetStart+k] = sourceStart+k;
    }
  }

  return (0);
}

/**************************************************************************
  \brief  Creates an instance of the tonality correction parameter module.
**************************************************************************/
int CreateTonCorrParamExtr (int chan, SBR_TON_CORR_EST *hTonCorr, int fs, int usb, int highBandStartSb, int channelOffset,
			    unsigned char *v_k_master, int numMaster, int ana_max_level, unsigned char *freqBandTable[2], int* nSfb,
			    int noiseBands, unsigned int useSpeechConfig)
{
  int i;

  memset(hTonCorr,0,sizeof(SBR_TON_CORR_EST));

  hTonCorr->prevTransientFlag  = 0;
  hTonCorr->transientNextFrame = 0;

  for(i=0;i<NO_OF_ESTIMATES;i++) {
    hTonCorr->quotaMatrix[i] = &(sbr_quotaMatrix[chan* NO_OF_ESTIMATES*QMF_CHANNELS + i*64]);
    memset(hTonCorr->quotaMatrix[i] ,0, sizeof(float)*QMF_CHANNELS);
  }

  /* Reset the patch.*/
  hTonCorr->guard = 0;
  hTonCorr->shiftStartSb = 1;

  if(resetPatch(hTonCorr, highBandStartSb, channelOffset, v_k_master, numMaster, fs)) {
    return(1);
  }

  CreateSbrNoiseFloorEstimate (&hTonCorr->sbrNoiseFloorEstimate, ana_max_level, freqBandTable[LO], nSfb[LO],
                               noiseBands, useSpeechConfig);

  CreateInvFiltDetector(&hTonCorr->sbrInvFilt, hTonCorr->sbrNoiseFloorEstimate.freqBandTableQmf,
                        hTonCorr->sbrNoiseFloorEstimate.noNoiseBands, useSpeechConfig);

  CreateSbrMissingHarmonicsDetector (chan, &hTonCorr->sbrMissingHarmonicsDetector);

  return (0);
}

