/*
  General tonality correction detector module.
*/
#ifndef _TON_CORR_EST_H
#define _TON_CORR_EST_H

#include "sbr_def.h"

#define MAX_NUM_PATCHES 6
#define NF_SMOOTHING_LENGTH 4
#define INVF_SMOOTHING_LENGTH 2
#define MAX_FREQ_COEFFS 27
#define MAX_ENVELOPES 5
#define NO_OF_ESTIMATES 4

typedef struct
{
  INVF_MODE regionSpace[5][5];
  INVF_MODE regionSpaceTransient[5][5];
}DETECTOR_PARAMETERS;

typedef struct
{
  float  origQuotaMean[INVF_SMOOTHING_LENGTH+1];
  float  sbrQuotaMean[INVF_SMOOTHING_LENGTH+1];
  float origQuotaMeanFilt;
  float sbrQuotaMeanFilt;
  float avgNrg;
}DETECTOR_VALUES;

typedef struct
{
  int prevRegionSbr[MAX_NUM_NOISE_VALUES];
  int prevRegionOrig[MAX_NUM_NOISE_VALUES];

  float nrgAvg;

  int freqBandTableInvFilt[MAX_NUM_NOISE_VALUES];
  int noDetectorBands;
  int noDetectorBandsMax;

  DETECTOR_PARAMETERS *detectorParams;
  INVF_MODE prevInvfMode[MAX_NUM_NOISE_VALUES];
  DETECTOR_VALUES detectorValues[MAX_NUM_NOISE_VALUES];
} SBR_INV_FILT_EST;

typedef struct
{
  float prevNoiseLevels[MAX_NUM_NOISE_VALUES][NF_SMOOTHING_LENGTH];
  int freqBandTableQmf[MAX_NUM_NOISE_VALUES + 1];
  float ana_max_level;
  float weightFac;
  int noNoiseBands;
  int noiseBands;
  float noiseFloorOffset[MAX_NUM_NOISE_VALUES];
  INVF_MODE diffThres;
} SBR_NOISE_FLOOR_ESTIMATE;

typedef struct
{
  float guideVectorDiff[32];
  float guideVectorOrig[32];
  unsigned char guideVectorDetected[32];
}GUIDE_VECTORS;

typedef struct
{
  int previousTransientFlag;
  int previousTransientFrame;
  int previousTransientPos;

  unsigned char guideScfb[MAX_FREQ_COEFFS];
  unsigned char* detectionVectors[NO_OF_ESTIMATES];
  float tonalityDiff[NO_OF_ESTIMATES][MAX_FREQ_COEFFS];
  float sfmOrig[NO_OF_ESTIMATES][MAX_FREQ_COEFFS];
  float sfmSbr[NO_OF_ESTIMATES][MAX_FREQ_COEFFS];
  GUIDE_VECTORS guideVectors[NO_OF_ESTIMATES];
} SBR_MISSING_HARMONICS_DETECTOR;

/** parameter set for one single patch */
typedef struct {
  int    sourceStartBand;
  int    sourceStopBand;
  int    guardStartBand;

  int    targetStartBand;
  int    targetBandOffs;
  int    numBandsInPatch;
} PATCH_PARAM;

typedef struct
{
  int frameStartIndex;
  int startIndexMatrix;
  int frameStartIndexInvfEst;

  int prevTransientFlag;
  int transientNextFrame;
  int transientPosOffset;

  float *quotaMatrix[NO_OF_ESTIMATES];
  float nrgVector[NO_OF_ESTIMATES];
  char indexVector[QMF_CHANNELS];

  PATCH_PARAM  patchParam[MAX_NUM_PATCHES];
  int    guard;
  int    shiftStartSb;
  int    noOfPatches;

  SBR_MISSING_HARMONICS_DETECTOR sbrMissingHarmonicsDetector;
  SBR_NOISE_FLOOR_ESTIMATE sbrNoiseFloorEstimate;
  SBR_INV_FILT_EST sbrInvFilt;
} SBR_TON_CORR_EST;

/* Functions */

void MissingHarmonicsDetectorQmf(SBR_MISSING_HARMONICS_DETECTOR *h_sbrMissingHarmonicsDetector, float ** pQuotaBuffer,
                                 char *indexVector, int* pAddHarmonicsFlag, unsigned char* pAddHarmonicsScaleFactorBands,
                                 const unsigned char* freqBandTable, int nSfb);

int CreateSbrMissingHarmonicsDetector(int chan, SBR_MISSING_HARMONICS_DETECTOR *h_sbrMissingHarmonicsDetector);

void NoiseFloorEstimateQmf (SBR_NOISE_FLOOR_ESTIMATE *h_sbrNoiseFloorEstimate,
                   int noNoiseBands, float *noiseLevels,
                   float** quotaMatrixOrig, char* indexVector,
                   INVF_MODE* pInvFiltLevels);

int CreateSbrNoiseFloorEstimate (SBR_NOISE_FLOOR_ESTIMATE *h_sbrNoiseFloorEstimate, int ana_max_level,
                                 const unsigned char *freqBandTable, int nSfb, int noiseBands, unsigned int useSpeechConfig);

void InverseFilteringDetectorQmf(SBR_INV_FILT_EST *hInvFilt, float ** quotaMatrix, float *nrgVector, char * indexVector,
                                 int transientFlag, INVF_MODE* infVec);

int CreateInvFiltDetector(SBR_INV_FILT_EST *hInvFilt, int* freqBandTableDetector, int numDetectorBands, unsigned int useSpeechConfig);

void TonCorrParamExtr(SBR_TON_CORR_EST *hTonCorr, INVF_MODE* infVec, float* noiseLevels,
					  int* missingHarmonicFlag, unsigned char * missingHarmonicsIndex,
                      int nNoiseEnvelopes, unsigned char * freqBandTable, int nSfb);

int CreateTonCorrParamExtr (int chan, SBR_TON_CORR_EST *hTonCorr, int fs, int usb, int highBandStartSb, 
			    int channelOffset, unsigned char *v_k_master, int numMaster, int ana_max_level,
			    unsigned char *freqBandTable[2], int* nSfb, int noiseBands, unsigned int useSpeechConfig);

#endif

